@@ -29,7 +29,9 @@ CORS_OPTS = [
     cfg.ListOpt('allowed_origin',
                 default=None,
                 help='Indicate whether this resource may be shared with the '
-                     'domain received in the requests "origin" header.'),
+                     'domain received in the requests "origin" header. '
+                     'Format: "<protocol>://<host>[:<port>]", no trailing '
+                     'slash. Example: https://horizon.example.com'),
     cfg.BoolOpt('allow_credentials',
                 default=True,
                 help='Indicate that the actual request can include user '
@@ -63,6 +65,11 @@ class InvalidOriginError(Exception):
             'CORS request from origin \'%s\' not permitted.' % origin)
 
 
+class _NoContentTypeResponse(webob.response.Response):
+
+    default_content_type = None  # prevents webob assigning content type
+
+
 class CORS(base.ConfigurableMiddleware):
     """CORS Middleware.
 
@@ -190,8 +197,8 @@ class CORS(base.ConfigurableMiddleware):
             for origin in allowed_origin:
 
                 if origin in self.allowed_origins:
-                    LOG.warn('Allowed origin [%s] already exists, skipping' % (
-                        allowed_origin,))
+                    LOG.warning('Allowed origin [%s] already exists, skipping'
+                                % (allowed_origin,))
                     continue
 
                 self.allowed_origins[origin] = {
@@ -285,7 +292,7 @@ class CORS(base.ConfigurableMiddleware):
         # underlying middleware's response content needs to be persisted.
         # Otherwise, create a new response.
         if 200 > response.status_code or response.status_code >= 300:
-            response = webob.response.Response(status=webob.exc.HTTPOk.code)
+            response = _NoContentTypeResponse(status=webob.exc.HTTPOk.code)
 
         # Does the request have an origin header? (Section 6.2.1)
         if 'Origin' not in request.headers:
@@ -389,7 +396,10 @@ class CORS(base.ConfigurableMiddleware):
             return
 
         # Set the default origin permission headers. (Sections 6.1.3 & 6.4)
-        response.headers['Vary'] = 'Origin'
+        if 'Vary' in response.headers:
+            response.headers['Vary'] += ',Origin'
+        else:
+            response.headers['Vary'] = 'Origin'
         response.headers['Access-Control-Allow-Origin'] = origin
 
         # Does this CORS configuration permit credentials? (Section 6.1.3)
